From 616b6a6d06602b65ec15b0e21dbd467046ce2085 Mon Sep 17 00:00:00 2001
From: Changhui Liu <liuchanghui@linuxdeepin.com>
Date: Sun, 15 Feb 2015 22:43:45 +0800
Subject: comctl32: SysDateTimePick32 add Alt+down hot key like windows
To: wine-patches <wine-patches@winehq.org>
Reply-To: wine-devel <wine-devel@winehq.org>

---
 dlls/comctl32/datetime.c       | 13 ++++++++++++
 dlls/comctl32/tests/datetime.c | 47 ++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 60 insertions(+)

diff --git a/dlls/comctl32/datetime.c b/dlls/comctl32/datetime.c
index 5c73a90..8483e65 100644
--- a/dlls/comctl32/datetime.c
+++ b/dlls/comctl32/datetime.c
@@ -1668,6 +1668,19 @@ DATETIME_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
     case WM_SETTEXT:
         return CB_ERR;
 
+    case WM_SYSKEYDOWN:
+        if (wParam == VK_DOWN)
+        {
+            POINT pt;
+            RECT rect;
+            GetClientRect(hwnd, &rect);
+            pt.x = rect.right - GetSystemMetrics(SM_CXVSCROLL)/2;
+            pt.y = rect.top + GetSystemMetrics(SM_CYHSCROLL)/2;
+
+            return DATETIME_LButtonDown (infoPtr, (SHORT)pt.x, (SHORT)pt.y);
+        }
+        /*else continue to DefWindowProcW*/
+
     default:
 	if ((uMsg >= WM_USER) && (uMsg < WM_APP) && !COMCTL32_IsReflectedMessage(uMsg))
 		ERR("unknown msg %04x wp=%08lx lp=%08lx\n",
diff --git a/dlls/comctl32/tests/datetime.c b/dlls/comctl32/tests/datetime.c
index 195c719..3305a7b 100644
--- a/dlls/comctl32/tests/datetime.c
+++ b/dlls/comctl32/tests/datetime.c
@@ -782,6 +782,52 @@ static void test_dts_shownone(void)
     DestroyWindow(hwnd);
 }
 
+
+DWORD WINAPI CheckMonthCalThread (void* param)
+{
+    int i = 0;
+    HWND hWndMonthCal = NULL;
+
+    for (i=0; i<5; ++i)
+    {
+        Sleep(500); /*wait the SysMonthCal32 control to show*/
+
+        hWndMonthCal = FindWindowA("SysMonthCal32", NULL);
+        if (NULL != hWndMonthCal)
+        {
+            trace("monthcal=%p\n", hWndMonthCal);
+            ok(IsWindowVisible(hWndMonthCal), "Expected MonthCal to be visible\n");
+
+            /*select a day by click point(0x10, 0x10)*/
+            PostMessageA(hWndMonthCal, WM_LBUTTONDOWN, 1, MAKEWPARAM(0x10,0x10));
+
+            *(HWND*)param = hWndMonthCal;
+
+            return TRUE;
+        }
+    }
+
+    ok(0, "Expected MonthCal to be valid\n");
+
+    return 0;
+}
+
+static void test_dtm_hotkey(void)
+{
+    HANDLE thread = 0;
+    HWND hWndMonthCal = 0;
+    HWND hWnd = create_datetime_control(0);
+    ok(hWnd != NULL, "Expected window to be valid\n");
+    ShowWindow(hWnd, SW_NORMAL);
+
+    thread = CreateThread (NULL, 0, CheckMonthCalThread, (void*)&hWndMonthCal, 0, 0);
+
+    SendMessageA(hWnd, WM_SYSKEYDOWN, VK_DOWN, 0);
+    WaitForSingleObject(thread, INFINITE);
+
+    DestroyWindow(hWnd);
+}
+
 START_TEST(datetime)
 {
     HMODULE hComctl32;
@@ -811,4 +857,5 @@ START_TEST(datetime)
     test_dtm_set_and_get_systemtime_with_limits();
     test_wm_set_get_text();
     test_dts_shownone();
+    test_dtm_hotkey();
 }
-- 
2.2.0

